==> mapsemble-1.0.0.tar.gz: mapsemble/src/MapsembleApi.php <==
<?php

declare(strict_types=1);

namespace Drupal\mapsemble;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Render\Renderer;
use Drupal\Core\Routing\UrlGeneratorInterface;
use Drupal\Core\Security\DoTrustedCallbackTrait;
use Drupal\Core\Site\Settings;
use Drupal\geofield\GeoPHP\GeoPHPWrapper;
use Drupal\mapsemble\Entity\MapsembleMap;
use GuzzleHttp\Client;

/**
 * Provides an API service for interacting with Mapsemble.
 */
final class MapsembleApi {
  use DoTrustedCallbackTrait;

  /**
   * The map storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected EntityStorageInterface $mapStorage;

  /**
   * The base URL for the Mapsemble app.
   */
  const MAPSEMBLE_APP_BASE = 'https://app.mapsemble.com';

  /**
   * The base URL for the Mapsemble embed.
   */
  const MAPSEMBLE_EMBED_BASE = 'https://embed.mapsemble.com';

  /**
   * Constructs a new MapsembleApi object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Render\Renderer $renderer
   *   The renderer service.
   * @param \Drupal\geofield\GeoPHP\GeoPHPWrapper $geoPHP
   *   The GeoPHP wrapper service.
   * @param \Drupal\Core\Routing\UrlGeneratorInterface $urlGenerator
   *   The URL generator service.
   */
  public function __construct(
    private readonly ConfigFactoryInterface $configFactory,
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly Renderer $renderer,
    private readonly GeoPHPWrapper $geoPHP,
    private readonly UrlGeneratorInterface $urlGenerator,
  ) {
    $this->mapStorage = $this->entityTypeManager->getStorage('mapsemble_map');
  }

  /**
   * Retrieves remote maps from Mapsemble.
   *
   * @return array
   *   An array of remote maps.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function getRemoteMaps(): array {
    $client = new Client([
      'base_uri' => Settings::get('mapsemble_app_base') ?? self::MAPSEMBLE_APP_BASE,
    ]);
    $apiKey = $this->configFactory->get('mapsemble.settings')->get('api_key');

    $request = $client->request('GET', '/api/maps', [
      'headers' => [
        'Authorization' => 'Bearer ' . $apiKey,
      ],
    ]);
    $content = json_decode($request->getBody()->getContents(), TRUE);
    $maps = [];
    foreach ($content as $map) {
      foreach ($map['layers'] as $layer) {
        if ($layer['type'] === 'vector:drupal') {
          $maps[$map['id']] = $map;
          break;
        }
      }
    }

    return $maps;
  }

  /**
   * Retrieves local maps from Drupal.
   *
   * @return array
   *   An array of local maps.
   */
  public function getLocalMaps(): array {
    return $this->entityTypeManager->getStorage('mapsemble_map')->loadMultiple();
  }

  /**
   * Retrieves a local map entity by its remote map ID.
   *
   * @param string $id
   *   The remote map ID.
   *
   * @return \Drupal\mapsemble\Entity\MapsembleMap
   *   The local map entity.
   */
  protected function getLocalMapEntityByRemoteMapId(string $id): MapsembleMap {
    $maps = $this->mapStorage->loadByProperties(['mapId' => $id]);
    $map = reset($maps);
    assert($map instanceof MapsembleMap);

    return $map;
  }

  /**
   * Retrieves data entities from a map.
   *
   * @param \Drupal\mapsemble\Entity\MapsembleMap $map
   *   The map entity.
   *
   * @return array
   *   An array of data entities.
   */
  protected function getDataEntitiesFromMap(MapsembleMap $map): array {
    $entityTypeId = $map->get('config')['entity_type_id'] ?? '';
    $bundle = $map->get('config')['bundle'] ?? '';
    if ($entityTypeId && $bundle) {
      $entityStorage = $this->entityTypeManager->getStorage($entityTypeId);
      $definition = $this->entityTypeManager->getDefinition($entityTypeId);
      if ($definition->hasKey('bundle')) {
        return $entityStorage->loadByProperties([$definition->getKey('bundle') => $bundle]);
      }
      else {
        return $entityStorage->loadMultiple();
      }
    }

    return [];
  }

  /**
   * Syncs all data to Mapsemble.
   */
  public function syncAllData(): void {
    $remoteMaps = $this->getRemoteMaps();
    foreach ($remoteMaps as $map) {
      $map = $this->getLocalMapEntityByRemoteMapId($map['id']);
      $entities = $this->getDataEntitiesFromMap($map);
      foreach ($entities as $entity) {
        $this->syncEntity($entity, $map);
      }
    }
  }

  /**
   * Gets a configured HTTP client.
   *
   * @return \GuzzleHttp\Client
   *   The HTTP client.
   */
  protected function getClient(): Client {
    return new Client([
      'base_uri' => Settings::get('mapsemble_app_base') ?? self::MAPSEMBLE_APP_BASE,
    ]);
  }

  /**
   * Deletes an entity from Mapsemble.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to delete.
   * @param \Drupal\mapsemble\Entity\MapsembleMap $map
   *   The map entity.
   */
  public function deleteEntity(EntityInterface $entity, MapsembleMap $map): void {
    $client = $this->getClient();
    $dataSourceId = $map->get('dataSourceId');

    try {
      $client->request('DELETE', '/api/data-source/' . $dataSourceId . '/data', [
        'json' => [
          'type' => 'FeatureCollection',
          'features' => [
            [
              'type' => 'Feature',
              'id' => $entity->id(),
            ],
          ],
        ],
        'headers' => [
          'Authorization' => 'Bearer ' . $this->getApiKey(),
        ],
      ]);
    }
    catch (\Exception $e) {
      // Do nothing.
    }
  }

  /**
   * Gets the API key.
   *
   * @return string
   *   The API key.
   */
  protected function getApiKey(): string {
    return $this->configFactory->get('mapsemble.settings')->get('api_key');
  }

  /**
   * Gets the Mapsemble content URL.
   *
   * @param string $routeName
   *   The route name.
   * @param \Drupal\mapsemble\Entity\MapsembleMap $map
   *   The map entity.
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   *
   * @return string
   *   The content URL.
   */
  protected function getMapsembleContentUrl(string $routeName, MapsembleMap $map, EntityInterface $entity): string {
    return $this->urlGenerator->generate(
      $routeName,
      [
        'mapsemble_map' => $map->id(),
        'entity_type' => $entity->getEntityTypeId(),
        'entity' => $entity->id(),
      ],
      UrlGeneratorInterface::ABSOLUTE_URL
    );
  }

  /**
   * Syncs a single entity to Mapsemble.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to sync.
   * @param \Drupal\mapsemble\Entity\MapsembleMap $map
   *   The map entity.
   */
  public function syncEntity(EntityInterface $entity, MapsembleMap $map): void {
    $client = $this->getClient();
    $dataSourceId = $map->get('dataSourceId');
    $dataViewMode = $map->get('config')['data_view_mode'] ?? 'default';
    $location_field = $map->get('config')['location_field'] ?? '';

    if (!$location_field) {
      return;
    }
    if (!$entity->hasField($location_field)) {
      return;
    }

    if (!$entity->get($location_field)->value) {
      return;
    }

    $build = $this->entityTypeManager->getViewBuilder($entity->getEntityTypeId())
      ->view($entity, $dataViewMode);
    foreach ($build['#pre_render'] as $callback) {
      $build = $this->doTrustedCallback($callback, [$build], '');
    }

    $data = [];
    foreach (Element::children($build) as $key) {
      $data[$key] = $this->renderer->renderInIsolation($build[$key]);
    }

    $data['_popup'] = $this->getMapsembleContentUrl('mapsemble.popup', $map, $entity);
    $data['_card'] = $this->getMapsembleContentUrl('mapsemble.card', $map, $entity);

    $feature = [
      'type' => 'Feature',
      'id' => $entity->id(),
      'geometry' => json_decode(
        $this->geoPHP->load($entity->get($location_field)->value, 'wkt')->out('json'),
        TRUE
      ),
      'properties' => $data,
    ];

    $client->request('PUT', '/api/data-source/' . $dataSourceId . '/data', [
      'json' => [
        'type' => 'FeatureCollection',
        'features' => [$feature],
      ],
      'headers' => [
        'Authorization' => 'Bearer ' . $this->getApiKey(),
      ],
    ]);
  }

}

==> mapsemble-1.0.2.tar.gz: mapsemble/src/MapsembleApi.php <==
<?php

declare(strict_types=1);

namespace Drupal\mapsemble;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Render\Renderer;
use Drupal\Core\Routing\UrlGeneratorInterface;
use Drupal\Core\Security\DoTrustedCallbackTrait;
use Drupal\Core\Site\Settings;
use Drupal\geofield\GeoPHP\GeoPHPWrapper;
use Drupal\mapsemble\Entity\MapsembleMap;
use GuzzleHttp\Client;

/**
 * Provides an API service for interacting with Mapsemble.
 */
final class MapsembleApi {

  use DoTrustedCallbackTrait;

  /**
   * The map storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected EntityStorageInterface $mapStorage;

  /**
   * The base URL for the Mapsemble app.
   */
  const MAPSEMBLE_APP_BASE = 'https://app.mapsemble.com';

  /**
   * The base URL for the Mapsemble embed.
   */
  const MAPSEMBLE_EMBED_BASE = 'https://embed.mapsemble.com';

  /**
   * Constructs a new MapsembleApi object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Render\Renderer $renderer
   *   The renderer service.
   * @param \Drupal\geofield\GeoPHP\GeoPHPWrapper $geoPHP
   *   The GeoPHP wrapper service.
   * @param \Drupal\Core\Routing\UrlGeneratorInterface $urlGenerator
   *   The URL generator service.
   */
  public function __construct(
    private readonly ConfigFactoryInterface $configFactory,
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly Renderer $renderer,
    private readonly GeoPHPWrapper $geoPHP,
    private readonly UrlGeneratorInterface $urlGenerator,
  ) {
    $this->mapStorage = $this->entityTypeManager->getStorage('mapsemble_map');
  }

  /**
   * Retrieves remote maps from Mapsemble.
   *
   * @return array
   *   An array of remote maps.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function getRemoteMaps(): array {
    $client = new Client([
      'base_uri' => Settings::get('mapsemble_app_base') ?? self::MAPSEMBLE_APP_BASE,
    ]);
    $apiKey = $this->configFactory->get('mapsemble.settings')->get('api_key');

    $request = $client->request('GET', '/api/maps', [
      'headers' => [
        'Authorization' => 'Bearer ' . $apiKey,
      ],
    ]);
    $content = json_decode($request->getBody()->getContents(), TRUE);
    $maps = [];
    foreach ($content as $map) {
      foreach ($map['layers'] as $layer) {
        if ($layer['type'] === 'vector:drupal') {
          $maps[$map['id']] = $map;
          break;
        }
      }
    }

    return $maps;
  }

  /**
   * Retrieves local maps from Drupal.
   *
   * @return array
   *   An array of local maps.
   */
  public function getLocalMaps(): array {
    return $this->entityTypeManager->getStorage('mapsemble_map')->loadMultiple();
  }

  /**
   * Retrieves a local map entity by its remote map ID.
   *
   * @param string $id
   *   The remote map ID.
   *
   * @return \Drupal\mapsemble\Entity\MapsembleMap
   *   The local map entity.
   */
  protected function getLocalMapEntityByRemoteMapId(string $id): MapsembleMap {
    $maps = $this->mapStorage->loadByProperties(['mapId' => $id]);
    $map = reset($maps);
    assert($map instanceof MapsembleMap);

    return $map;
  }

  /**
   * Retrieves data entities from a map.
   *
   * @param \Drupal\mapsemble\Entity\MapsembleMap $map
   *   The map entity.
   *
   * @return array
   *   An array of data entities.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getDataEntitiesFromMap(MapsembleMap $map): array {
    $entityTypeId = $map->get('settings')['entity_type_id'] ?? '';
    $bundle = $map->get('settings')['bundle'] ?? '';
    if ($entityTypeId && $bundle) {
      $entityStorage = $this->entityTypeManager->getStorage($entityTypeId);
      $definition = $this->entityTypeManager->getDefinition($entityTypeId);
      if ($definition->hasKey('bundle')) {
        return $entityStorage->loadByProperties([$definition->getKey('bundle') => $bundle]);
      }
      else {
        return $entityStorage->loadMultiple();
      }
    }

    return [];
  }

  /**
   * Syncs all data to Mapsemble.
   */
  public function syncAllData(): void {
    $remoteMaps = $this->getRemoteMaps();
    foreach ($remoteMaps as $map) {
      $map = $this->getLocalMapEntityByRemoteMapId($map['id']);
      $entities = $this->getDataEntitiesFromMap($map);
      foreach ($entities as $entity) {
        $this->syncEntity($entity, $map);
      }
    }
  }

  /**
   * Gets a configured HTTP client.
   *
   * @return \GuzzleHttp\Client
   *   The HTTP client.
   */
  protected function getClient(): Client {
    return new Client([
      'base_uri' => Settings::get('mapsemble_app_base') ?? self::MAPSEMBLE_APP_BASE,
    ]);
  }

  /**
   * Deletes an entity from Mapsemble.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to delete.
   * @param \Drupal\mapsemble\Entity\MapsembleMap $map
   *   The map entity.
   */
  public function deleteEntity(EntityInterface $entity, MapsembleMap $map): void {
    $client = $this->getClient();
    $dataSourceId = $map->get('dataSourceId');

    try {
      $client->request('DELETE', '/api/data-source/' . $dataSourceId . '/data', [
        'json' => [
          'type' => 'FeatureCollection',
          'features' => [
            [
              'type' => 'Feature',
              'id' => $entity->id(),
            ],
          ],
        ],
        'headers' => [
          'Authorization' => 'Bearer ' . $this->getApiKey(),
        ],
      ]);
    }
    catch (\Exception $e) {
      // Do nothing.
    }
  }

  /**
   * Gets the API key.
   *
   * @return string
   *   The API key.
   */
  protected function getApiKey(): string {
    return $this->configFactory->get('mapsemble.settings')->get('api_key');
  }

  /**
   * Gets the Mapsemble content URL.
   *
   * @param string $routeName
   *   The route name.
   * @param \Drupal\mapsemble\Entity\MapsembleMap $map
   *   The map entity.
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   *
   * @return string
   *   The content URL.
   */
  protected function getMapsembleContentUrl(string $routeName, MapsembleMap $map, EntityInterface $entity): string {
    return $this->urlGenerator->generate(
      $routeName,
      [
        'mapsemble_map' => $map->id(),
        'entity_type' => $entity->getEntityTypeId(),
        'entity' => $entity->id(),
      ],
      UrlGeneratorInterface::ABSOLUTE_URL
    );
  }

  /**
   * Syncs a single entity to Mapsemble.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to sync.
   * @param \Drupal\mapsemble\Entity\MapsembleMap $map
   *   The map entity.
   */
  public function syncEntity(EntityInterface $entity, MapsembleMap $map): void {
    $client = $this->getClient();
    $dataSourceId = $map->get('dataSourceId');
    $dataViewMode = $map->get('settings')['data_view_mode'] ?? 'default';
    $location_field = $map->get('settings')['location_field'] ?? '';

    if (!$location_field) {
      return;
    }
    if (!$entity->hasField($location_field)) {
      return;
    }

    if (!$entity->get($location_field)->value) {
      return;
    }

    $build = $this->entityTypeManager->getViewBuilder($entity->getEntityTypeId())
      ->view($entity, $dataViewMode);
    foreach ($build['#pre_render'] as $callback) {
      $build = $this->doTrustedCallback($callback, [$build], '');
    }

    $data = [];
    foreach (Element::children($build) as $key) {
      $data[$key] = $this->renderer->renderInIsolation($build[$key]);
    }

    $data['_popup'] = $this->getMapsembleContentUrl('mapsemble.popup', $map, $entity);
    $data['_card'] = $this->getMapsembleContentUrl('mapsemble.card', $map, $entity);

    $feature = [
      'type' => 'Feature',
      'id' => $entity->id(),
      'geometry' => json_decode(
        $this->geoPHP->load($entity->get($location_field)->value, 'wkt')->out('json'),
        TRUE
      ),
      'properties' => $data,
    ];

    $client->request('PUT', '/api/data-source/' . $dataSourceId . '/data', [
      'json' => [
        'type' => 'FeatureCollection',
        'features' => [$feature],
      ],
      'headers' => [
        'Authorization' => 'Bearer ' . $this->getApiKey(),
      ],
    ]);
  }

}
